 SET all_types              / monogastrics, ruminants, cereals, wheat, maize, rice, oilcrops, pulses, starchyRoots, energycrops, pasture /;
 
 SET crop(all_types)                                          / wheat, maize, rice, oilcrops, pulses, starchyRoots, energycrops, pasture /;
 SET crop_less_pasture(crop)                                  / wheat, maize, rice, oilcrops, pulses, starchyRoots, energycrops/;
 SET cereal_crop(crop)                                        / wheat, maize, rice /;
 SET non_cereal_crop(crop)                                                        / oilcrops, pulses, starchyRoots, energycrops, pasture /;
 SET feed_crop(crop)                                          / wheat, maize,       oilcrops,                                   pasture/;
 SET feed_crop_less_pasture(feed_crop)                        / wheat, maize,       oilcrops          /;
 SET not_feed_crop(crop)                                                    / rice,           pulses, starchyRoots, energycrops /;
 SET import_crop(all_types) / monogastrics, ruminants,          wheat, maize, rice, oilcrops, pulses, starchyRoots, energycrops /;
 
 SET location;
 PARAMETER suitableLandArea(location)        areas of land in Mha;
 PARAMETER previousArea(crop, location)      areas for previous timestep in Mha;
 PARAMETER previousFertIntensity(crop, location);
 PARAMETER previousIrrigIntensity(crop, location); 
 PARAMETER previousOtherIntensity(crop, location);
 PARAMETER previousRuminantFeed(crop);
 PARAMETER previousMonogastricFeed(crop);
 PARAMETER previousImportAmount(all_types);
 PARAMETER previousExportAmount(all_types);
 PARAMETER yieldNone(crop, location)         yield in t per ha;
 PARAMETER yieldFertOnly(crop, location)     yield in t per ha;
 PARAMETER yieldIrrigOnly(crop, location)    yield in t per ha;
 PARAMETER yieldBoth(crop, location)         yield in t per ha;
 PARAMETER fertParam(crop, location)         yield response to fertilizer parameter;
 PARAMETER irrigParam(crop, location)        yield response to irrigation parameter;
 PARAMETER demand(all_types)                 in t;
 PARAMETER exportPrices(import_crop)         prices for exports;
 PARAMETER importPrices(import_crop)         prices for imports;
 PARAMETER maxNetImport(import_crop)         maximum net import for each crop based on world market;
 PARAMETER minNetImport(import_crop)         minimum net import for each crop based on world market;
 PARAMETER irrigCost(location)               irrigation cost in cost per 1000 Mlitre or   Mha for each litre per m2;
 PARAMETER irrigMaxRate(crop, location)      max water application rate irrigation in litre per m2;
 PARAMETER irrigConstraint(location)         max water available for irrigation in litre per m2;
 
 PARAMETER agriExpansionCost(location)       price for increase agricultural area varies based on amount of managed or unmanaged forest;
 SCALAR pastureIncCost                       price for increasing pasture area;
 SCALAR cropIncCost                          price for increasing crop;
 SCALAR cropDecCost                          price for decreasing crop;
 SCALAR pastureDecCost                       price for decreasing pasture;
  
 SCALAR meatEfficency                        efficiency of converting feed and pasture into animal products;
 SCALAR fertiliserUnitCost                   fert cost at max fert rate;
 SCALAR otherIParam                          yield response to other intensity;
 SCALAR otherICost                           cost of other intensity;
 SCALAR unhandledCropArea                    includes fruit veg forage crops set aside and failed crop;
 SCALAR domesticPriceMarkup                  factor price increased from cost of production;
 SCALAR maxLandExpansionRate                 max rate of country land expansion 

*$gdxin "/Users/peteralexander/Documents/R_Workspace/UNPLUM/temp/GamsTmp/t1.gdx"
$gdxin %gdxincname% 
$load location, suitableLandArea, demand, agriExpansionCost, cropIncCost, pastureIncCost, cropDecCost, pastureDecCost
$load previousArea, previousFertIntensity, previousIrrigIntensity, previousOtherIntensity, previousRuminantFeed, previousMonogastricFeed, previousImportAmount, previousExportAmount
$load yieldNone, yieldFertOnly, yieldIrrigOnly, yieldBoth
$load fertParam, irrigParam, otherIParam, exportPrices, importPrices, maxNetImport, minNetImport, unhandledCropArea, maxLandExpansionRate
$load meatEfficency, otherICost, irrigCost, irrigMaxRate, irrigConstraint, fertiliserUnitCost, domesticPriceMarkup
$gdxin    
 
 SCALAR delta use to smooth power function see 7.5 www.gams.com dd docs solversconopt.pdf / 0.00000000001 /;
     
 SCALAR minDemandPerCereal / 0.0 /;
 
 previousArea(crop_less_pasture, location) = previousArea(crop_less_pasture, location) * (1.0 - unhandledCropArea);
 
 PARAMETER cropDM(crop)  kg DM per kg of feed the conversion from feed to meat is done in the R animal product index.  Pasture is in DM terms already
          /   wheat         0.87   
              maize         0.86  
              rice          0.89
              oilcrops      0.88  
              pulses        0.31
              starchyRoots  0.21
              pasture       1    / ; 
   
 PARAMETER prodCost(crop)  cost per ha before intensity values not including fertiliser or irrigation i.e. seed spray and other.  In 1000 $ per ha
          /   wheat             0.32   
              maize             0.26 
              rice              0.4
              oilcrops          0.24
              pulses            0.31
              starchyRoots      4.64 
              energyCrops       0.34 / ;

 PARAMETER baseCost(crop);
 PARAMETER otherIntCost(crop);
 baseCost(crop) = prodCost(crop)*0.35;
 otherIntCost(crop) = baseCost(crop)*0.65 + otherICost;
 baseCost('pasture') = 0.03;
 otherIntCost('pasture') = 0.8 + otherICost;
                              
 VARIABLES
       area(crop, location)               total area for each crop - Mha
       fertI(crop, location)              fertilizer intensity for each crop - factor between 0 and 1
       irrigI(crop, location)             irrigation intensity for each crop - factor between 0 and 1
       otherIntensity(crop, location)
       feedAmount(crop)                   amount of feed for each crop - Mt
       ruminantFeed(feed_crop)            amount of feed for ruminant animals - Mt
       monogastricFeed(feed_crop)         amount of feed for monogatric animals - Mt
       importAmount(all_types)            imports of crops and meat - Mt
       exportAmount(all_types)            exports of crops and meat - Mt
       yield(crop, location)              yield per area for each crop - t per ha
       unitCost(crop, location)           cost per area for each crop - cost
       net_supply(crop)                   supply after exports and feed
       agriLandExpansion(location)        addition agricultural land needed as it must be positive it deliberately does not account for abandonment
       cropIncrease(location)             
       cropDecrease(location)             
       pastureIncrease(location)         
       pastureDecrease(location)          
       total_cost                         total cost of domestic supply including net imports;
 
 POSITIVE VARIABLE area, fertI, irrigI, otherIntensity, feedAmount, ruminantFeed, monogastricFeed, importAmount, exportAmount,
                   agriLandExpansion, cropIncrease, cropDecrease, pastureDecrease, pastureIncrease;
  
 EQUATIONS
       UNIT_COST_EQ(crop, location)                   cost per area
       YIELD_EQ(crop, location)                         yield given chosen intensity
       NON_CEREAL_DEMAND_CONSTRAINT(non_cereal_crop)    satisfy demand for non-cereal crop 
       CEREAL_DEMAND_CONSTRAINT(cereal_crop)            satisfy demand for cereal so that exports can at least be met. Could also allow min. proporation of cereal consumption of each type
       TOTAL_CEREAL_DEMAND_CONSTRAINT                   satisfy demand for cereal
       RUMINANT_DEMAND_CONSTRAINT                       satisfy demand for ruminant products
       MONOGASTRICS_DEMAND_CONSTRAINT                   satisfy demand for monogastric products
       TOTAL_FEED_CONSTRAINT(feed_crop)                 total feed constraint
       MAX_FERT_INTENSITY_CONSTRAINT(crop, location)    constraint on maximum fertilizer intensity
       MAX_IRRIG_INTENSITY_CONSTRAINT(crop, location)   constraint on maximum irrigation intensity
       MAX_OTHER_INTENSITY_CONSTRAINT(crop, location)
       TOTAL_LAND_CHANGE_CONSTRAINT(location)           constraint on suitable land use 
       NON_FEED_CROP_CONSTRAINT(not_feed_crop)          constraint to set non feed crop feed usage to zero
       MAX_NET_IMPORT_CONSTRAINT(import_crop)           constraint on max net imports
       MIN_NET_IMPORT_CONSTRAINT(import_crop)           constraint on min net imports
       PASTURE_IMPORT_CONSTRAINT                        constraint to not import pasture
       PASTURE_EXPORT_CONSTRAINT
       IRRIGATION_CONSTRAINT(location)                  constraint no water usage
       NET_SUPPLY_EQ(crop)                              calc net supply for crops
       AGRI_LAND_EXPANSION_CALC(location)               calc agriLandExpansion
       CROP_INCREASE_CALC(location)                     
       CROP_DECREASE_CALC(location)                     
       PASTURE_INCREASE_CONV_CALC(location)             
       PASTURE_DECREASE_CONV_CALC(location)          
       PASTURE_TOTAL_CHANGE_CONSTRAINT(location)   
       LAND_RATE_CHANGE_CONSTRAINT_INCREASE
       COST_EQ                                        total cost objective function;
 
 UNIT_COST_EQ(crop, location) .. unitCost(crop, location) =E=  (     baseCost(crop) +           
                                                                     fertiliserUnitCost * fertI(crop, location) + 
                                                                     irrigCost(location) * irrigMaxRate(crop, location) * irrigI(crop, location) +
                                                                     otherIntCost(crop) * otherIntensity(crop, location)
                                                                     ) ;
 
 YIELD_EQ(crop, location) .. yield(crop, location) =E= (
               yieldNone(crop, location) + 
               (yieldFertOnly(crop, location) - yieldNone(crop, location)) * (1 - exp(-fertParam(crop, location)*fertI(crop, location))) +
               (yieldIrrigOnly(crop, location) - yieldNone(crop, location)) * (1 - exp(-irrigParam(crop, location)*irrigI(crop, location))) +
               (yieldBoth(crop, location) + yieldNone(crop, location) - yieldFertOnly(crop, location) - yieldIrrigOnly(crop, location)) *
                                      (1 - exp(-fertParam(crop, location)*fertI(crop, location))) * (1 - exp(-irrigParam(crop, location)*irrigI(crop, location)))
            ) * (1 - exp(-otherIntensity(crop, location)*otherIParam));
   
 NET_SUPPLY_EQ(crop) .. net_supply(crop) =E= sum(location, area(crop, location) * yield(crop, location)) - feedAmount(crop) + importAmount(crop) - exportAmount(crop);
  
 NON_CEREAL_DEMAND_CONSTRAINT(non_cereal_crop) .. net_supply(non_cereal_crop) =G= demand(non_cereal_crop);
  
 CEREAL_DEMAND_CONSTRAINT(cereal_crop) .. net_supply(cereal_crop) =G= demand('cereals') * minDemandPerCereal;
  
 TOTAL_CEREAL_DEMAND_CONSTRAINT .. sum(cereal_crop, net_supply(cereal_crop)) =G= demand('cereals');
 
 RUMINANT_DEMAND_CONSTRAINT .. meatEfficency*sum(feed_crop, ruminantFeed(feed_crop) * cropDM(feed_crop)) =G= demand('ruminants') - importAmount('ruminants') + exportAmount('ruminants');
    
 MONOGASTRICS_DEMAND_CONSTRAINT .. meatEfficency*sum(feed_crop_less_pasture, monogastricFeed(feed_crop_less_pasture) * cropDM(feed_crop_less_pasture)) =G= demand('monogastrics') - importAmount('monogastrics') + exportAmount('monogastrics');
    
 TOTAL_FEED_CONSTRAINT(feed_crop) .. feedAmount(feed_crop) =E= ruminantFeed(feed_crop) + monogastricFeed(feed_crop);
 
 MAX_FERT_INTENSITY_CONSTRAINT(crop, location) .. fertI(crop, location) =L= 1;
 MAX_IRRIG_INTENSITY_CONSTRAINT(crop, location) .. irrigI(crop, location) =L= 1;
 MAX_OTHER_INTENSITY_CONSTRAINT(crop, location) .. otherIntensity(crop, location) =L= 1;
 
 TOTAL_LAND_CHANGE_CONSTRAINT(location) .. suitableLandArea(location) =G= sum(crop_less_pasture, area(crop_less_pasture, location)) / (1.0 - unhandledCropArea) + area('pasture', location);
  
 NON_FEED_CROP_CONSTRAINT(not_feed_crop) .. feedAmount(not_feed_crop) =E= 0;
 
 MAX_NET_IMPORT_CONSTRAINT(import_crop) .. importAmount(import_crop) - exportAmount(import_crop) =L= maxNetImport(import_crop);
 MIN_NET_IMPORT_CONSTRAINT(import_crop) .. importAmount(import_crop) - exportAmount(import_crop) =G= minNetImport(import_crop);
 PASTURE_IMPORT_CONSTRAINT .. importAmount('pasture') =E= 0;
 PASTURE_EXPORT_CONSTRAINT ..  exportAmount('pasture') =E= 0;
  
 IRRIGATION_CONSTRAINT(location) .. irrigConstraint(location) * suitableLandArea(location) =G= sum(crop, irrigMaxRate(crop, location) * irrigI(crop, location) * area(crop, location));

 AGRI_LAND_EXPANSION_CALC(location) .. agriLandExpansion(location) =G= sum(crop, area(crop, location) - previousArea(crop, location)); 
 
 CROP_INCREASE_CALC(location) .. cropIncrease(location) =G= sum(crop_less_pasture, area(crop_less_pasture, location) - previousArea(crop_less_pasture, location));
 CROP_DECREASE_CALC(location) .. cropDecrease(location) =G= -sum(crop_less_pasture, area(crop_less_pasture, location) - previousArea(crop_less_pasture, location));
 PASTURE_INCREASE_CONV_CALC(location) .. pastureIncrease(location) =G= area('pasture', location) - previousArea('pasture', location);
 PASTURE_DECREASE_CONV_CALC(location) .. pastureDecrease(location) =G= -(area('pasture', location) - previousArea('pasture', location));
 PASTURE_TOTAL_CHANGE_CONSTRAINT(location) .. pastureIncrease(location) -pastureDecrease(location) =G= area('pasture', location) - previousArea('pasture', location);
  
 LAND_RATE_CHANGE_CONSTRAINT_INCREASE .. sum(location, cropIncrease(location)) / sum(location, (suitableLandArea(location)*(1.0 - unhandledCropArea))-sum(crop, previousArea(crop, location))) =L= maxLandExpansionRate;

 COST_EQ .. total_cost =E= 
         ( 
              (   SUM((crop, location), area(crop, location) * unitCost(crop, location)) +
                  
                  SUM(location, 
                     agriExpansionCost(location) * agriLandExpansion(location) +
                     cropIncCost * cropIncrease(location) + 
                     pastureIncCost * pastureIncrease(location) + 
                     cropDecCost * cropDecrease(location) + 
                     pastureDecCost * pastureDecrease(location)
                     )
               ) * domesticPriceMarkup + 

               SUM(import_crop, importAmount(import_crop) * importPrices(import_crop) - exportAmount(import_crop) * exportPrices(import_crop)) 
         );
 
 MODEL LAND_USE /ALL/ ;
 fertI.L(crop, location) = previousFertIntensity(crop, location);
 irrigI.L(crop, location) = previousIrrigIntensity(crop, location);
 otherIntensity.L(crop, location) = previousOtherIntensity(crop, location);
 area.L(crop, location) = previousArea(crop, location);
 ruminantFeed.L(feed_crop) = previousRuminantFeed(feed_crop);
 monogastricFeed.L(feed_crop) = previousMonogastricFeed(feed_crop);
 importAmount.L(all_types) = previousImportAmount(all_types);
 exportAmount.L(all_types) = previousExportAmount(all_types);
  
 SOLVE LAND_USE USING NLP MINIMIZING total_cost;
       
 display agriLandExpansion.L, previousArea, irrigMaxRate, otherIntensity.L, fertI.L, irrigI.L, area.L, cropIncrease.L, cropDecrease.L, pastureIncrease.L, pastureDecrease.L;
* display suitableLandArea, area.l, net_supply.l, demand, feedAmount.l, yield.l, importAmount.l, exportAmount.l, unitCost.l, exportPrices, maxNetImport, minNetImport;

* Calculate summary information used in Java process 
 parameter totalProd(all_types);
 parameter totalProdCost(all_types);
 parameter totalArea(crop);
 parameter totalCropland(location);
 parameter netImportAmount(all_types);
 parameter netImportCost(all_types);
 parameter feedCost(all_types);
 
* Production quantities based on smaller area (before unhandledCropArea adjustment applied)
 totalProd(crop) = sum(location, area.l(crop, location) * yield.l(crop, location));
 totalProd('ruminants') = meatEfficency*sum(feed_crop, ruminantFeed.l(feed_crop) * cropDM(feed_crop));
 totalProd('monogastrics') = meatEfficency*sum(feed_crop, monogastricFeed.l(feed_crop) * cropDM(feed_crop));
 
* Cost based on adjusted area
 area.l(crop_less_pasture, location) = area.l(crop_less_pasture, location) / (1.0 - unhandledCropArea);
 totalProdCost(crop) = sum(location, unitCost.l(crop, location) * area.l(crop, location));
 totalCropland(location) = sum(crop_less_pasture, area.l(crop_less_pasture, location));
 totalArea(crop) = sum(location, area.l(crop, location));
 
 netImportCost(import_crop) = importAmount.l(import_crop) * importPrices(import_crop) - exportAmount.l(import_crop) * exportPrices(import_crop);
 netImportAmount(import_crop) = importAmount.l(import_crop) - exportAmount.l(import_crop);
         
 Scalar ms 'model status', ss 'solve status'; 
 ms=LAND_USE.modelstat;
 ss=LAND_USE.solvestat;
