package ac.ed.lurg.country;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.demand.BaseConsumpManager;
import ac.ed.lurg.utils.CollectionHelper;
import ac.ed.lurg.utils.StringTabularReader;

public class CompositeCountryManager {

	private Map<String, CompositeCountry> mapFromName = new HashMap<String, CompositeCountry>();
	private Map<SingleCountry, CompositeCountry> mapFromSingleCountry;
	private Map<CompositeCountry, List<SingleCountry>> mapFromCompositeCountry;
	private StringTabularReader countryGroups;
	
	public CompositeCountryManager(BaseConsumpManager baseConsumpManager) {
		super();
		if (ModelConfig.PREDEFINED_COUNTRY_GROUPING) {
			countryGroups = new StringTabularReader(",", new String[]{"Country", "PlumGroup"});
			countryGroups.read(ModelConfig.COUNTRY_GROUPING_FILE);
		}
		populate(baseConsumpManager);
	}
	
	private String getCountryGroup(SingleCountry aCountry) {		
		Map<String, String> queryMap = new HashMap<String, String>();
		queryMap.put("Country", aCountry.getCountryName());
		Map<String, String> row = countryGroups.querySingle(queryMap);
		return row.get("PlumGroup");
	}
	
	private void populate(BaseConsumpManager baseConsumpManager) {
		mapFromSingleCountry = new HashMap<SingleCountry, CompositeCountry>();
		
		HashSet<String> countryExclusionList = new HashSet<String>(Arrays.asList("Philippines"));
		
		for (SingleCountry c : baseConsumpManager.getAllCountries()) {
			CompositeCountry cc;
			
			if (ModelConfig.PREDEFINED_COUNTRY_GROUPING)
				cc = getForName(getCountryGroup(c));
			else if (baseConsumpManager.getPopForGrouping(c) < ModelConfig.POPULATION_AGGREG_LIMIT || countryExclusionList.contains(c.getCountryName()))
				cc = getForName(c.getRegion() + "_other");
			else 
				cc = getForName(c.getCountryName());
			
			mapFromSingleCountry.put(c, cc);
		}
		
		mapFromCompositeCountry = CollectionHelper.invertMap(mapFromSingleCountry);
	}

	private CompositeCountry getForName(String name) {
		CompositeCountry cc = mapFromName.get(name);
		
		if (cc == null) {
			cc = new CompositeCountry(name);
			mapFromName.put(name, cc);
		}
		return cc;
	}
	
	public CompositeCountry getForSingleCountry(SingleCountry c) {
		return mapFromSingleCountry.get(c);
	}
	
	public Collection<CompositeCountry> getAll() {
		return mapFromName.values();
	}
	
	public Collection<SingleCountry> getAllForCompositeCountry(CompositeCountry cc) {
		return mapFromCompositeCountry.get(cc);
	}
}
