package ac.ed.lurg.country.gams;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CountryPrice;
import ac.ed.lurg.country.ImportExportConstraint;
import ac.ed.lurg.landuse.CropUsageData;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.CropType;

public class GamsCountryInput {

	private CompositeCountry country; // not really required but useful for debugging
	private Map<CommodityType, Double> projectedDemand;
	private Map<CropType, ImportExportConstraint> importConstraints;
	private Map<CropType, CountryPrice> countryPrices;
	private Map<CropType, CropUsageData> previousCropUsageData;

	public GamsCountryInput(CompositeCountry country, Map<CommodityType, Double> projectedDemand, Map<CropType, CountryPrice> countryPrices, 
			Map<CropType, ImportExportConstraint> importConstraints, Map<CropType, CropUsageData> previousCropUsageData) {
		super();
		this.country = country;
		this.projectedDemand = projectedDemand;
		this.importConstraints = importConstraints;
		this.countryPrices = countryPrices;
		this.previousCropUsageData = previousCropUsageData;
	}
	
	public GamsCountryInput(GamsCountryInput gamsInput) {
		this(gamsInput.country, gamsInput.projectedDemand, gamsInput.countryPrices, gamsInput.importConstraints, gamsInput.previousCropUsageData);
	}
	
	public CompositeCountry getCountry() { 
		return country;
	}
	
	public Map<CommodityType, Double> getProjectedDemand() {
		return projectedDemand;
	}

	public Map<CropType, CountryPrice> getCountryPrices() {
		return countryPrices;
	}
	
	public Map<CropType, ImportExportConstraint> getImportConstraints() {
		return importConstraints;
	}
	
	public Map<CropType, CropUsageData> getPreviousCropUsageData() {
		return previousCropUsageData;
	}

	public double getMeatEfficiency() {
		return ModelConfig.MEAT_EFFICIENCY;  // this is already handled by the feed conversion efficiency for each animal product
	}

	public Map<CropType, Double> getMinNetImport() {
		Map<CropType, Double> netImport = new HashMap<CropType, Double>();
		for (Map.Entry<CropType, ImportExportConstraint> entry : importConstraints.entrySet()) {
			netImport.put(entry.getKey(), entry.getValue().getMinNetImport());
		}
		return netImport;
	}

	public Map<CropType, Double> getMaxNetImport() {
		Map<CropType, Double> netImport = new HashMap<CropType, Double>();
		for (Map.Entry<CropType, ImportExportConstraint> entry : importConstraints.entrySet()) {
			netImport.put(entry.getKey(), entry.getValue().getMaxNetImport());
		}
		return netImport;
	}
}