package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.Timestep;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.utils.LogWriter;

public abstract class AbstractDemandManager {

	protected CompositeCountryManager compositeCountryManager;
	protected BioenergyDemandManager bioenergyDemandManager;

	public AbstractDemandManager(CompositeCountryManager compositeCountryManager) {
		this.compositeCountryManager = compositeCountryManager;
		bioenergyDemandManager  = new BioenergyDemandManager();
	}

	public Map<CommodityType, Double> getDemand(CompositeCountry cc, int year) {

		if (!ModelConfig.CHANGE_DEMAND_YEAR) 
			year = ModelConfig.BASE_YEAR;

		Map<CommodityType, Double> demandMap = new HashMap<CommodityType, Double>();
				
		for (SingleCountry c : compositeCountryManager.getAllForCompositeCountry(cc)) {
			Map<CommodityType, Double> foodDemandMap = getFoodDemand(c, year);

			for (CommodityType commodity : CommodityType.getAllItems()) {
				Double food = foodDemandMap.get(commodity);
				double bioenergy = bioenergyDemandManager.getFirstGenBioenergyDemand(c, year, commodity);
				//		LogWriter.println(String.format("Country %s comm %s: %f",  country, commodity, bioenergy));

				double newDemand = ((food==null ? 0.0 : food.doubleValue()) + bioenergy) * (1.0 + commodity.getSeedWasteRate());
				Double prevDemand = demandMap.get(commodity);

				if (prevDemand != null)
					newDemand += prevDemand.doubleValue();  // aggregate if composite country

				demandMap.put(commodity, newDemand);
			}
		}

		return demandMap;
	}
	
	abstract Map<CommodityType, Double> getFoodDemand(SingleCountry c, int year);

	public double getSecondGenBioenergyDemand(Timestep timestep) {
		int year = ModelConfig.CHANGE_DEMAND_YEAR ? timestep.getYear() : ModelConfig.BASE_YEAR;
		double d = bioenergyDemandManager.getSecondGenBioenergyDemand(year);
		LogWriter.println("Global gen2 bioenergy demand in " + timestep + " is " + d);
		return d;
	}
}
