package ac.ed.lurg.types;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;

public enum CommodityType {

	CEREALS("Cereals", "cereals", false),
	OILCROPS("Oilcrops", "oilcrops", false),
	PULSES("Pulses", "pulses", false),
	STARCHY_ROOTS("Starchy Roots", "starchyRoots", false),
	MONOGASTRICS("Monogastrics", "monogastrics", true),
	RUMINANTS("Ruminants", "ruminants", true);

	private String faoName;
	private String gamsName;
	private boolean isAnimalProduct;
	
	CommodityType (String faoName, String gamsName, boolean isAnimalProduct) {
		this.faoName = faoName;
		this.gamsName = gamsName;
		this.isAnimalProduct = isAnimalProduct;
	}
	
	private static final Map<String, CommodityType> faoNameCache = new HashMap<String, CommodityType>();
    static {
        for (CommodityType c : values()) {
        	faoNameCache.put(c.getFaoName(), c);
       }
    }

	public static CommodityType getForFaoName(String faoName) {
		CommodityType commodity = faoNameCache.get(faoName);

		if (commodity == null) 
			throw new RuntimeException("Can't find CommodityType for faoName: " + faoName);
		
		return commodity;
	}
	
	public static Collection<CommodityType> getAllItems() {
		return faoNameCache.values();
	}

	public String getFaoName() {
		return faoName;
	}
	
	public String getGamsName() {
		return gamsName;
	}

	public boolean isAnimalProduct() {
		return isAnimalProduct;
	}
	
	public double getSeedWasteRate() {
		// just waste rate for animal products
		return isAnimalProduct ? ModelConfig.SEED_AND_WASTE_FRACTION/2 : ModelConfig.SEED_AND_WASTE_FRACTION;
	}
}
