package ac.ed.lurg.yield;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.types.YieldType;

public class YieldResponse {
	Map<YieldType, Double> yields = new HashMap<YieldType, Double>();
	private double fertParm;
	private double irrigParm;

	private static double defaultParam;
	
	static {
		defaultParam =  calcParam(0, 0.8, 1, 0, 0.5, 1.0);  // default assumes 80% at mid point
	}

	
	public void setYield(YieldType type, double yield) {
		yields.put(type, yield);
		fertParm = 0;
		irrigParm = 0;
	}
	
	public double getYield(YieldType type) {
		Double d = yields.get(type);
		
		if (d == null) {
		//	LogWriter.printlnError("YieldResponse: No yield found for type " + type);
			return Double.NaN;
		}
		
		return d.doubleValue();
	}		
	
	public double getFertParam() {
		if (fertParm == 0) {
			//	fertParm = calcParam (0, 0.5, 1, 5.0/200, 50.0/200, 200.0/200); // we do have MID fert data, but looks wrong 
			fertParm = calcParam(getYield(YieldType.NO_FERT_NO_IRRIG), getYield(YieldType.FERT_MID_NO_IRRIG), getYield(YieldType.FERT_MAX_NO_IRRIG), 
					ModelConfig.MIN_FERT_AMOUNT/ModelConfig.MAX_FERT_AMOUNT, ModelConfig.MID_FERT_AMOUNT/ModelConfig.MAX_FERT_AMOUNT, ModelConfig.MAX_FERT_AMOUNT/ModelConfig.MAX_FERT_AMOUNT);
			
			//LogWriter.println(String.format("%s %s %s", getYield(YieldType.NO_FERT_NO_IRRIG), getYield(YieldType.FERT_MID_NO_IRRIG), getYield(YieldType.FERT_MAX_NO_IRRIG)));
		}
		return fertParm;
	}
	
	public double getIrrigParam() {
		if (irrigParm == 0) {
			irrigParm = defaultParam; 
		}

		return irrigParm;
	}
	
	private static double calcParam(double yMin, double yMid, double yMax, double xMin, double xMid, double xMax) {
		if (yMid <= yMin || yMax <= yMid)
			return defaultParam;

		return -Math.log(1 - ((yMid-yMin)/(yMax-yMin))) * (xMax-xMin) / (xMid-xMin); 					// 1-exp(-x) function
	//	return Math.log((yMid - yMin)/(yMax - yMin)) / Math.log((xMid - xMin)/(xMax - xMin)) / xMax;  	// Cobb-Douglas
	}
	
}